/*
 *				**** SONG LIST ****
 *
 * This file contains routines that read the file songlist.txt
 * and provides various ways to access the data
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "alblist.h"

static int verbose = 5;	/* verbose switch */
static char *Prog=NULL;		/* program name */
static int ncells=4;	/* how many albums per row */

static ProdAPtrList *SongTitle = NULL;	/* songs by title */
static ProdAPtrList *SongKey = NULL;  /* songs by code */

static void read_songlist(void);	/* ingest songlist.txt */
int read_line(char *buf, FILE *fp);
struct song *create_song(void);

void set_verbose_songlist(int val) { verbose = val; }

void
songlist(char *pname)
{
	Prog = pname;
   if (verbose > 5) printf("%s: Initializing songlist\n",
   	Prog);

	read_songlist();	/* ingest songlist.txt */
   if (verbose > 0) printf("%s: %d songs read\n",
   							Prog, SongTitle->nitems);
}

int
get_number_songs(void)
{
	if (SongTitle == NULL) return -1;
   return SongTitle->nitems;
}

char **
get_song_titles(void)
{
	if (SongTitle == NULL) return NULL;
   return SongTitle->key;
}

char **
get_song_keys(void)
{
	if (SongKey == NULL) return NULL;
   return SongKey->key;
}

void
read_songlist(void)
{
	FILE *fp;
   char buf[BUFSIZ];
   struct song *song;
   char *flds[NUM_SONG_FLDS+1];
   int nflds;

   if (SongTitle != NULL)
   {
   	fprintf(stderr, "%s read_songlist called more than once!\n", Prog);
      return;
   }

   SongTitle = create_aptr_prodlist(200);
   SongKey = create_aptr_prodlist(200);

   if (SongTitle == NULL || SongKey == NULL)
   {
   	fprintf(stderr, "%s: Can't malloc memory for Song Lists!\n", Prog);
      if (SongTitle != NULL)
      {
      	free(SongTitle);
         SongTitle = NULL;
      }
      if (SongKey != NULL)
      {
      	free(SongKey);
         SongKey = NULL;
      }
   }

   if ((fp = fopen("songlist.txt", "r")) == NULL)
   {
   	fprintf(stderr, "%s: Can't open songlist.txt\n", Prog);
      return;
   }

   while (read_line(buf, fp) == 0)
   { /* read lines and store them in Song Lists */
   	if (*buf == '\0') continue;	/* blank line */
      if (*buf == '*')
      { /* alias - not handled yet */
      	fprintf(stderr, "%s: Can't handle alias yet!\n", Prog);
         return;
      }

      /* line is title, clips, lyrics, code, comments */
      nflds = split_flds(buf, flds, '|', 0, NUM_SONG_FLDS+1);
      if (nflds <= SONG_FLD_KEY)
      {
      	fprintf(stderr, "%s: Not enough fields (%d) for Song!\n", Prog, nflds);
         continue;
      }

      /* valid song like - create structure to store it */
      if ((song = create_song()) == NULL)
      {
      	fprintf(stderr, "%s: Can't malloc memory for song!\n", Prog);
         return;
      }

      /* we know that we at least have up to song code */
      song->title = strdup(flds[SONG_FLD_TITLE]);
      song->clips = *flds[SONG_FLD_CLIPS] == 'y';
      song->lyrics = *flds[SONG_FLD_LYRICS] == 'y';
      song->key = strdup(flds[SONG_FLD_KEY]);
      if (nflds > SONG_FLD_COMMENT)
      {
      	song->comments = strdup(flds[SONG_FLD_COMMENT]);
      } else song->comments = NULL;
      if (nflds > NUM_SONG_FLDS)
      { /* extra junk on the end */
      	fprintf(stderr, "%s: Extra data at end of song line ignored (%s)\n",
         	Prog, flds[NUM_SONG_FLDS]);
      }

      /* song structure build - add to both indecies */
      bupdate_aptr_prodlist(SongTitle, song->title, song);
      bupdate_aptr_prodlist(SongKey, song->key, song);
   }

   return;
}

struct song*
create_song(void)
{
	struct song *song;

	song = (struct song *)malloc(sizeof(struct song));
   if (song != NULL)
   {
   	memset(song, '\0', sizeof(struct song));
      song->albs = create_alpha_prodlist(5);
   }
   return song;
}

void
create_songs(FILE *fp)
{
	int lett, nsong, ncell;
   struct song *song;

   /* generate title */
   fprintf(fp, "Song Links\n\n");

   /*
    * We are about to build a table containing
    * the alphabet & the number sign (#)
    * Each cell will be a link to a list of
    * songs starting with that letter
    *
    * nsong will be used to index the SongTitle array
    * If no song starts with a given letter,
    * no link will be created, but the cell will be there
    */

   /* create index table for songs */
	fprintf(fp, "<table align=center border=2>\n");
   nsong = 0; /* first entry in the SongTitle index */
   song = (struct song *)SongTitle->item[nsong]; /* first song */
   if (isdigit(song->title[0]))
   { /* song starts with digit */
   	fprintf(fp, "<td bgcolor=#ffff00><a href=#0>#</a></td>\n");
		for (; nsong < SongTitle->nitems; nsong++)
      {
      	song=(struct song *)SongTitle->item[nsong];
         if (! isdigit(song->title[0])) break;
      }
   }
   else
   { /* no songs start with digit */
   	fprintf(fp, "<td bgcolor=#c0c0c0>#</td>\n");
   }

   /* now add letters */
   for (lett='A'; lett <= 'Z'; lett++)
   { /* create index table # & A-Z */
		if (song->title[0] == lett)
      { /* song starts with this letter */
      	fprintf(fp, "<td bgcolor=#ffff00><a href=#%c>%c</a></td>\n",
         		lett, lett);
         for ( ; nsong < SongTitle->nitems; nsong++)
         { /* find song startingn with a different letter */
         	song = (struct song *)SongTitle->item[nsong];
            if (song->title[0] != lett) break;
         }
      }
      else
      { /* no song starts with this letter */
      	fprintf(fp, "<td bgcolor=#c0c0c0>%c</td>\n", lett);
      }
      /* break row if needed */
      /* if ((lett-'A'+1) % 10 == 9) fprintf(fp, "</tr><tr>\n"); */
   }
   fprintf(fp, "</tr></table>\n");

   /* Now generate a table for each unique letter */
   nsong = 0;
   song = (struct song *)SongTitle->item[nsong];
   if (isdigit(song->title[0]))
   { /* we have songs starting with a digit */
   	fprintf(fp, "<a name=0><hr></a><h1>Songs Starting with a Number</h1>\n");
      fprintf(fp, "<table align=center border=2>\n");
		ncell = 0;	/* how many cells we have created */
      for (; nsong < SongTitle->nitems; nsong++, ncell++)
      { /* loop until we find a non-digit */
         if (ncell % ncells == 0) fprintf(fp, "<tr>\n");
      	song = (struct song *)SongTitle->item[nsong];
         if (!isdigit(song->title[0])) break;
         fprintf(fp, "<td align=center><a href=../songs/%s.html#beg>%s</a></td>\n",
						song->key, song->title);
         if (ncell%ncells == (ncells-1)) fprintf(fp, "</tr>\n");
      }
      /* add end of row if needed */
      if (ncell % ncells != 0) fprintf(fp, "</tr>\n");
      fprintf(fp, "</table>\n");	/* end of digit table */
      fprintf(fp, ">Top\n");	/* lik to top of page */
   }

   /* now go thru alphabet */
   for (lett = 'A'; lett <= 'Z'; lett++)
   { /* create table for each letter that starts a song */
   	if (song->title[0] != lett) continue;	/* no songs w/ letter */
      fprintf(fp, "<a name=%c><hr></a>\n<h1>Songs that Start with %c</h1>\n",
      	lett, lett);
      fprintf(fp, "<table align=center border=2>\n");
      ncell = 0;	/* no cells yet */
      for (; nsong < SongTitle->nitems; nsong++, ncell++)
      {
      	song = (struct song *)SongTitle->item[nsong];
         if (song->title[0] != lett) break; /* done with letter */
         if (ncell % ncells == 0) fprintf(fp, "<tr>\n");
         fprintf(fp, "<td align=center><a href=../songs/%s.html#beg>%s</a></td>\n",
         	song->key, song->title);
         if (ncell%ncells == (ncells-1)) fprintf(fp, "</tr>\n");
      }
      if (ncell%ncells != 0) fprintf(fp, "</tr>\n");
      fprintf(fp, "</table>\n");
      fprintf(fp, ">Top\n");	/* lik to top of page */
   } /* end of loop through songs */
}

void
create_songs_links(FILE *fp)
{
	int lett, nsong, ncell;
   struct song *song;

   /* generate title */
   fprintf(fp, "Song Links\n\n");

   /*
    * We are about to build a table containing
    * the alphabet & the number sign (#)
    * Each cell will be a link to a list of
    * songs starting with that letter
    *
    * nsong will be used to index the SongTitle array
    * If no song starts with a given letter,
    * no link will be created, but the cell will be there
    */

   /* create index table for songs */
	fprintf(fp, "<table align=center border=2>\n");
   nsong = 0; /* first entry in the SongTitle index */
   song = (struct song *)SongTitle->item[nsong]; /* first song */
   if (isdigit(song->title[0]))
   { /* song starts with digit */
   	fprintf(fp, "<td bgcolor=#ffff00><a href=#0>#</a></td>\n");
		for (; nsong < SongTitle->nitems; nsong++)
      {
      	song=(struct song *)SongTitle->item[nsong];
         if (! isdigit(song->title[0])) break;
      }
   }
   else
   { /* no songs start with digit */
   	fprintf(fp, "<td bgcolor=#c0c0c0>#</td>\n");
   }

   /* now add letters */
   for (lett='A'; lett <= 'Z'; lett++)
   { /* create index table # & A-Z */
		if (song->title[0] == lett)
      { /* song starts with this letter */
      	fprintf(fp, "<td bgcolor=#ffff00><a href=#%c>%c</a></td>\n",
         		lett, lett);
         for ( ; nsong < SongTitle->nitems; nsong++)
         { /* find song startingn with a different letter */
         	song = (struct song *)SongTitle->item[nsong];
            if (song->title[0] != lett) break;
         }
      }
      else
      { /* no song starts with this letter */
      	fprintf(fp, "<td bgcolor=#c0c0c0>%c</td>\n", lett);
      }
      /* break row if needed */
      if ((lett-'A'+1) % 5 == 4) fprintf(fp, "</tr><tr>\n");
   }
   fprintf(fp, "</tr></table>\n");

   /* Now generate a list for each unique letter */
   nsong = 0;
   song = (struct song *)SongTitle->item[nsong];
   if (isdigit(song->title[0]))
   { /* we have songs starting with a digit */
   	fprintf(fp, "<a name=0><hr></a><h4>Number</h4>\n");
      for (; nsong < SongTitle->nitems; nsong++, ncell++)
      { /* loop until we find a non-digit */
      	song = (struct song *)SongTitle->item[nsong];
         if (!isdigit(song->title[0])) break;
         fprintf(fp, "<li><a href=../songs/%s.html#beg target=det>%s</a><p>\n",
						song->key, song->title);
         if (ncell%ncells == (ncells-1)) fprintf(fp, "</tr>\n");
      }
      fprintf(fp, ">Top\n");	/* lik to top of page */
   }

   /* now go thru alphabet */
   for (lett = 'A'; lett <= 'Z'; lett++)
   { /* create table for each letter that starts a song */
   	if (song->title[0] != lett) continue;	/* no songs w/ letter */
      fprintf(fp, "<a name=%c><hr></a>\n<h4>%c Songs</h4>\n",
      	lett, lett);
      for (; nsong < SongTitle->nitems; nsong++, ncell++)
      {
      	song = (struct song *)SongTitle->item[nsong];
         if (song->title[0] != lett) break; /* done with letter */
         fprintf(fp, "<li><a href=../songs/%s.html#beg target=det>%s</a><p>\n",
         	song->key, song->title);
      }
      fprintf(fp, ">Top\n");	/* lik to top of page */
   } /* end of loop through songs */
}

struct song *
find_song_by_title(char *title)
{
	int loc;

   loc = bsearch_aptr_prodlist(SongTitle, title);
   if (loc < 0) return NULL;
	return (struct song *)SongTitle->item[loc];
}

struct song *
find_song_by_key(char *key)
{
	int loc;

   loc = bsearch_aptr_prodlist(SongKey, key);
   if (loc < 0) return NULL;
	return (struct song *)SongKey->item[loc];
}

struct song *
get_song_by_key_loc(int loc)
{
	if (loc < 0 || loc >= SongKey->nitems) return NULL;
   return SongKey->item[loc];
}

int
is_song_key(char *key)
{
	int loc;

   loc = bsearch_aptr_prodlist(SongKey, key);
   return (loc >= 0);
}

char *get_song_title_for_key(char *key)
{
	struct song *song;

   song = find_song_by_key(key);
   if (song == NULL) return NULL;
   return song->title;
}

char *get_song_key_for_title(char *title)
{
	struct song *song;

   song = find_song_by_title(title);
   if (song == NULL) return NULL;
   return song->key;
}

void
add_album_to_song(char *albkey, char *songkey)
{
	struct song *song;
   struct album *alb;

   if ((song = find_song_by_key(songkey)) != NULL)
   {
   	insert_sorted_alpha_prodlist(song->albs, albkey);
   }
}

void
display_song_title_ent(int loc)
{
	struct song *song;

   printf("Song title for entry %d is %s\n",
   	loc, SongTitle->key[loc]);
   song = (struct song *)SongTitle->item[loc];
   display_song_struct(song);
}

void
display_song_key_ent(int loc)
{
	struct song *song;

	printf("Song key for entry %d is %s\n", loc, SongKey->key[loc]);
   song = (struct song*)SongKey->item[loc];
   display_song_struct(song);
}

void
display_song_struct(struct song *song)
{
	int i;

	printf("key=%s, title=%s, lyrics=%d, clips=%d, nalb=%d\n",
   	song->key, song->title, song->lyrics, song->clips,
      song->albs->nitems);

   for (i=0; i < song->albs->nitems; i++)
   {
   	printf(" %s,", song->albs->item[i]);
   }
   if (song->albs->nitems > 0) printf(" [END]\n");
}


